% Find the nearest neighbor in a data set to some query element
function findNNtoQ()
    close all;
    clear all;
    
    % random walk settings
    dir = '../data/randomwalk/';
    mem_pages = 10;        
    queryNum = 5001;%9001;    
    page_size = 1000;    
    % lightcurves settings
%     dir = '../data/lightcurves/timeseries_2K_sorted.tts';    
%     mem_pages = 1;    
%     queryNum = 'OGLE052057.44-693317.7.Idia.folded';%'OGLE052906.15-693116.3.Idia.folded';%'OGLE051812.28-693625.1.Idia.folded';%'OGLE052057.44-693317.7.Idia.folded';
     
    [Q, queryNum] = loadQuery(queryNum, page_size, dir);
    nnQ = []; 
    nnDist = realmax;
    for i = 0:mem_pages-1
        disp(['----Page #' num2str(i+1) ' of ' num2str(mem_pages)]);
        [DB, Lbls] = loadDB(i, dir);
        if (mem_pages == 1) 
            excludeIndx = queryNum;
        else 
            if (floor(queryNum/page_size) == i) 
                excludeIndx = rem(queryNum, page_size);
            else
                excludeIndx = -1;
            end;
        end;
        [tmpNN, tmpNNDist] = findNN(DB, Q, nnDist, excludeIndx); 
        if (nnDist > tmpNNDist)
            nnDist = tmpNNDist;
            nnQ = tmpNN;
        end;
    end;
    figure;
    plot(Q, 'b', 'LineWidth', 2); 
    hold on;    
    plot(nnQ, 'r', 'LineWidth', 2);
    legend('Discord', 'NearestNbr');
    title(['NN to the discord. Dist=' num2str(nnDist)]);
    
function [NN, NNDist] = findNN(DB, Q, oldnnDist, excludeIndx)
    NNDist = oldnnDist;
    NN = [];
    for i = 1:size(DB,1)
        if (i == excludeIndx)
            continue;
        end;
        % todo: use the early abandoning when computing the distance
        tmpDist = computeDist(Q, DB(i,:));
        if (tmpDist < NNDist)
            NNDist = tmpDist;
            NN = DB(i,:);
        end;
    end;
    
function [Q, queryNum] = loadQuery(queryNum, page_size, dir)
    if (size(findstr(dir, 'lightcurves'),1) > 0) 
        [DB, Lbls] = loadDB1(dir);
        for i = 1:size(Lbls,1)
            if (size(findstr(queryNum, cell2mat(cell(Lbls(i)))),1) > 0)
               Q = DB(i,:);
               queryNum = i;
               return;
            end;
        end;
    else
        num = floor(queryNum/page_size);
        [DB, Lbls] = loadDB(num, dir);
        pos = rem(queryNum, page_size);
        Q = DB(pos, :);
        Lbl = queryNum;
    end;
    
function [DB, Lbls] = loadDB(num, dir)    
    if (size(findstr(dir, 'lightcurves'),1) > 0) 
        [DB, Lbls] = loadDB1(dir);
        return;
    end;
    fileName = [dir 'TS'];
    if (num < 10)
        suff = '000';
    elseif (num < 100)
        suff = '00';
    elseif (num < 1000)
        suff = '0';
    else
        suff = '';
    end;
    fileName = [fileName suff num2str(num) '.txt'];
    DB = load(fileName);
    Lbls = [];

function [DB, Lbls] = loadDB1(inFile)
    inFile1 = '';
    isomap_util = cluster.IsomapUtil;
    v = isomap_util.loadData(inFile);  
    DB = v.get(0);
    Lbls = v.get(1);
    
